/**
 * Sample code for "Untestable Code" serie. Find more on
 * http://kaczanowscy.pl/tomek.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including * without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package pl.kaczanowscy.uc.domain;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;
import pl.kaczanowscy.uc.domain.EntityX;
import pl.kaczanowscy.uc.domain.InvalidItemStatus;
import pl.kaczanowscy.uc.domain.ServiceA;
import pl.kaczanowscy.uc.domain.ServiceB;

/**
 * Version without mocks. ServiceB is subclassed for the purposes of this test.
 * 
 * @author Tomek Kaczanowski
 * 
 */
public class ServiceANoMockTest extends TestCase {

	private static final BigDecimal TOTAL = BigDecimal.TEN;

	class MyServiceA extends ServiceA {
		private boolean saved = false;
		private ServiceB serviceB;

		public void setServiceB(ServiceB serviceB) {
			this.serviceB = serviceB;
		}

		@Override
		ServiceB getServiceB() {
			return serviceB;
		}

		@Override
		List find(EntityX data) {
			return Collections.EMPTY_LIST;
		}

		@Override
		void save(EntityX data) {
			this.saved = true;
		}

		public boolean isSaved() {
			return saved;
		}

	};

	class MyServiceB extends ServiceB {

		private boolean computeTotalExecuted = false;

		@Override
		public BigDecimal computeTotal(List items) throws InvalidItemStatus {
			computeTotalExecuted = true;
			return TOTAL;
		}

		public boolean isComputeTotalExecuted() {
			return computeTotalExecuted;
		}

	}

	/** Test spy and SUT at the same time. */
	private MyServiceA serviceA;

	/** Test spy for serviceB collaborator. */
	private MyServiceB serviceB;

	@Override
	protected void setUp() throws Exception {
		serviceA = new MyServiceA();
		serviceB = new MyServiceB();
		serviceA.setServiceB(serviceB);
	}

	public void testBusinessOperation() throws InvalidItemStatus {
		EntityX entity = new EntityX(1, "abc", "def");
		serviceA.doBusinessOperationXyz(entity);
		assertEquals(TOTAL, entity.getTotal());
		assertTrue(serviceA.isSaved());
		assertTrue(serviceB.isComputeTotalExecuted());
	}
}
